package ucot.core;

import java.net.URL;
import java.util.Vector;

import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.output.OutputInterface;
import ucot.parser.ParserInterface;

public interface ControlInterface {
	/**
	 * Shuts down the program.
	 *
	 */
	void shutdown();
	
	/**
	 * Loads usecases from file
	 * @param url url of the file
	 */
	void loadUseCases(URL url);
	
	/**
	 * Loads analyze model from a file
	 * @param url url of the file containing analyze model
	 */
	void loadAnalyzeModel(URL url);
	
	/**
	 * Saves analyze model to a file
	 * @param url target file
	 */
	void saveAnalyzeModel(URL url);
	
	/**
	 * Returns all loaded usecases
	 * @return use case colletion
	 */
	UseCaseCollection getUseCaseCollection();
	
	/**
	 * Requests core to parse use case, perform heuristic on it and add it to given analyze model
	 * @param useCase use case to work magic on
	 * @param model analyze model where specified usecase is added
	 */
	void addToAnalyzeModel(UseCase usecase, AnalyzeModel model) throws Exception;
	
	/**
	 * returns vector of all analyze models from CORE
	 * @return specified analyze model
	 */
	Vector<AnalyzeModel> getAnalyzeModels();	
	
	/**
	 * Returns a vector of parsers that are available
	 * @return vector containing parser adapters
	 */
	Vector<ParserInterface> getParsers();
	
	/**
	 * Returns a list of heuristics that are available
	 * @return vector containing heuristic adapter
	 */
	Vector<HeuristicInterface> getHeuristics();
	
	/**
	 * Returns a list of output adapters available.
	 * @return vector containing output adapter names
	 */
	Vector<OutputInterface> getOutputs();
	
	/**
	 * Returns a list of input adapters available.
	 * @return vector containing input adapter names
	 */
	InputCollection getInputs();
	
	/**
	 * Sets the default parser to use.
	 * @param parser Parser to be used by default
	 */
	void setCurrentParser(ParserInterface parser);
	
	/**
	 * Sets the default heuristic to use.
	 * @param heuristic default heuristic to be used
	 */
	void setCurrentHeuristic(HeuristicInterface heuristic);
	
	/**
	 * returns the current default parser
	 * @return default parser
	 */
	ParserInterface getCurrentParser();
	
	/**
	 * returns the current heuristic
	 * @return current heuristic
	 */
	HeuristicInterface getCurrentheuristic();
	
	/**
	 * Exports given analyze model to given url using given outputadapter
	 * @param url destination url
	 * @param output output adapter
	 * @param model analyzemodel to export
	 */
	void output(URL url, OutputInterface output, AnalyzeModel model) throws Exception;	
}
