package ucot.model;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Observable;
import java.util.Queue;
import java.util.Set;

import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.Updation.UpdationType;

/**
 * This is the internal representation of 
 * analyzed use case. It is build up from 
 * two basic kind of objects: items and relations
 * between them.
 * 
 * 
 * 
 * @author pajumasu
 *
 */
public class AnalyzeModel extends Observable {
    private Set<Item> items = new HashSet<Item>();
    private UseCaseCollection usecases = new UseCaseCollection();
    private Set<Relation> relations = new HashSet<Relation>();
    ModelEditor editor;
    
    public AnalyzeModel(){
    	 editor = new ModelEditorImpl(this);
    }
    
    
    /**
     * Adds item in to the model
     * @param e Item to be added.
     */
    public void addItem(Item e) {
        if (items.add(e)){
        	setChanged();
        	notifyObservers(new Updation(UpdationType.NEW,e));
        }
    }
 
    public void addUseCase(UseCase a) {
    	usecases.addUseCase(a);
    }
    
    public UseCaseCollection getUseCaseCollection() {
    	return usecases;
    }

    public Entity getEntityByName(String name) {
    	Set<Item> entities = getItemsType("entity");
    	//Go trough the items
        for (Item item : entities) {        	
        	if (item.getName().equalsIgnoreCase(name))
        		return (Entity)item;            
        }
    	return null;
    }
    
    
    public Item getItemByName(String name) {
    	//Go trough the items
        for (Item item : items) {
        	
        	if (item.getName().equalsIgnoreCase(name))
        		return item;            
        }
    	return null;
    }
        
    /**
     * Returns items which type name is equal to given name.
     * @param typeName The type name we are searching for.
     * @return Set if items that match the given type name.
     */
    public Set<Item> getItemsType(String typeName) {
        Set<Item> ret = new HashSet<Item>();
        
        // Go trough all item
        for (Item item : items) {
            if (item.getTypeName().equals(typeName)){
                // Item type is right so return it.
                ret.add(item);                
            }
        }
        return ret;
    }

    /**
     * Extract entities from given thing.
     * If the thing is relation then all
     * both to and from things are futher
     * iteratively extracted.
     * If the thing is item then the item 
     * is added to this model.
     * @param i The thing to be investigated.
     */
    private void extractEntities(Thing i) {
        
        // Make queue for the items to be handled.
        Queue<Thing> tobeHandled = new LinkedList<Thing>();
        
        // Add the first item to the handling queue.
        tobeHandled.add(i);
        
        // Loop until the queue is empty.
        while (tobeHandled.size() > 0) {
            
            // Fetch first one from the queue.
            Thing current = tobeHandled.poll();
            
            if (current instanceof Item) {
                // It is simple item so add it.
                addItem((Item) current);
            } else if (current instanceof Relation) {
                // It is relation so add both ends to
                // the queue and add the relation
                // to this model.
                tobeHandled.add(((Relation) current).getFrom());
                tobeHandled.add(((Relation) current).getTo());
                relations.add((Relation) current);
            }
        }
    }

    /**
     * Returns relations that point to given thing.
     * @param item
     * @return
     */
    public Set<Relation> getRelationsTo(Thing item) {
        Set<Relation> ret = new HashSet<Relation>();

        // Go trhough all relations.
        for (Relation relation : relations) {

            if (relation.getTo().equals(item)) {
                // Given Thing is the from object of the relation
                // so return this relation.
                ret.add(relation);
            }
        }
        return ret;
    }

    /**
     * Returns relations that originates from given thing.
     * @param item
     * @return
     */
    public Set<Relation> getRelationsFrom(Thing item) {
        Set<Relation> ret = new HashSet<Relation>();

        // Go trhough all relations.
        for (Relation relation : relations) {
            if (relation.getFrom().equals(item)) {
                // Given Thing is the from object of the relation
                // so return this relation.
                ret.add(relation);
            }
        }
        return ret;
    }

    /**
     * Returns relations that are certain type
     * and are originated from given item.
     * 
     * getRelationsFrom(new Entity("user"), RelationType.CONTAINS);
     * Returns all the relations that connect
     * Things containded by the 'user' Entity. Which
     * means methods, attributes and such.
     * 
     * @param item The originator.
     * @param type The RelationType we are looking for.
     * @return
     */
    public Set<Relation> getRelationsFrom(Thing item, RelationType type) {
        // First get relations that originates from given thing.
        Set<Relation> ret = getRelationsFrom(item);

        // Next find out which satisfy the type condition.
        for (Iterator<Relation> i = ret.iterator(); i.hasNext();) {
            Relation relation = i.next();
            if (!relation.getType().equals(type)) {
                // The RelationType is wrong kind so dont return this relation
                i.remove();
            }
        }
        return ret;
    }

    /**
     * Returns relations that point to thing of spesific type
     * and are originated from given item.
     * 
     * getRelationsFrom(new Entity("user"), "method");
     * Returns all the relations that are originated from
     * Entity called 'user' and are pointing to Thing of
     * type 'method'. Simply this calls returns all the
     * Relations that connects the methods of the 'user' Entity.
     * 
     * @param item The originator.
     * @param toType The type of the Thing relation points.
     * @return
     */
    public Set<Relation> getRelationsFrom(Thing item, String toType) {
        // First get relations that originates from given thing.
        Set<Relation> ret = getRelationsFrom(item);

        // Next find out which satisfy the toType condition.
        for (Iterator<Relation> i = ret.iterator(); i.hasNext();) {
            Relation relation = i.next();
            if (!relation.getTo().getTypeName().equals(toType)) {
                // Type name is different from toType so dont return this.
                i.remove();
            }
        }
        return ret;
    }
    
    
    public Relation getRelation(Thing thing1, Method method, Thing thing2){
    	Set<Relation> relationsFrom = getRelationsFrom(thing1);
    	// Next find out which satisfy the conditions.
        for (Iterator<Relation> i = relationsFrom.iterator(); i.hasNext();) {
            Relation relation = i.next();
            if (relation.getTo().equals(thing2) && relation.getName().equals(method.name) && relation.getType().equals(RelationType.INFLUENCE)) {
            	return relation;
            }
        }
        return null;
    }
    
    public Set<Relation> getRelationsFrom(Thing thing1, Method method){
    	Set<Relation> relationsFrom = getRelationsFrom(thing1);
    	// Next find out which satisfy the conditions.
        for (Iterator<Relation> i = relationsFrom.iterator(); i.hasNext();) {
            Relation relation = i.next();
            if (! (relation.getName().equals(method.name) 
            		&& relation.getType().equals(RelationType.INFLUENCE))) {
            	i.remove();
            }
        }
        return relationsFrom;
    }

    
    public void removeItem(Item entity){
    	Collection<Relation> from = getRelationsFrom(entity);
    	Collection<Relation> to = getRelationsTo(entity);
    	for(Relation rel : from){
    		removeRelation(rel);
    	}
    	for(Relation rel : to){
    		removeRelation(rel);
    	}
    	items.remove(entity);
    	setChanged();
    	notifyObservers(new Updation(UpdationType.DELETION,entity));
    }
    
    
    protected synchronized void setChanged(){
    	super.setChanged();
    }
    public void removeRelation(Relation relation){
    	relations.remove(relation);
    	setChanged();
    	notifyObservers(new Updation(UpdationType.DELETION, relation));
    }


    /**
     * Adds relation to this model.
     * @param relation The relation beign added.
     */
    public void addRelation(Relation relation) {
        // Extract (and add) items and sub relations from the relation.
        extractEntities(relation);

        // Add the relation itself to the model.
        relations.add(relation);
        setChanged();
        notifyObservers(new Updation(UpdationType.NEW,relation));
    }

    public ModelEditor getEditor(){
    	return editor;
    }
    
}
