package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import ucot.model.AnalyzeModel;
import ucot.model.Entity;
import ucot.model.Item;
import ucot.model.Method;
import ucot.model.Relation;
import ucot.model.RelationType;
import ucot.output.Output;

/**
 * Dialog to modify analyzeModel.
 * 
 * usage:
 *   call method:
 *     modifyEntityProperties(Entity to edit, AnalyzeModel)
 *   wich returns the modified analyzeModel.
 * 
 * @author ilanliuk
 *
 */
public class EntityPropertiesDialog extends JDialog {
	
	/**
	 * This method creates a new JComboBox Cell Editor for a JTable element
	 * from the given String array. Duplicate entries from given array will
	 * be filtered and the array is sorted to alphabetical order.
	 * @param values Available options in the JComboBox.
	 * @return JComboBox table cell editor as a DefaultCellEditor.
	 */
	protected DefaultCellEditor createComboBoxCellEditor(String[] values) {
		JComboBox result = new JComboBox();
		List<String> valuesList = Arrays.asList(values);
		// Sort values to alphabetical order.
		Collections.sort(valuesList);
		// Add values to the JComboBox item list and filter out duplicates.
		for ( int i = 0; i < valuesList.size(); i++ )
			if ( i >= 0 && result.getItemCount() > 0 &&
					!valuesList.get(i).equals(result.getItemAt(result.getItemCount()-1)) )
				result.addItem(valuesList.get(i));
			else if ( result.getItemCount() == 0 )
				result.addItem(valuesList.get(i));
		// Spawn a new DefaultCellEditor from the resulting JComboBox.
		return new DefaultCellEditor(result);
	}

    private Entity selectedEntity;
    private AnalyzeModel model;
    private JTable methodsTable;
    private JPanel methodsContainerPanel;
    private JScrollPane methodsScrollPane;
    private JTable relationsTable;
    private JPanel relationsContainerPanel;
    private JScrollPane relationsScrollPane;
    private DefaultTableModel relationsTableModel;
    private final String[] relationsTableColumns = {"suhde"};
    private DefaultTableModel methodsTableModel;
    private final String[] methodsTableColumns = {"metodi", "kohde"};
    private JTextField entityName = new JTextField("Name");
    
    
    // Create actionlistener for buttons
    ActionListener buttonListener = new ActionListener() {

        public void actionPerformed(ActionEvent e) {
           
            if ( "MAIN_OK".equals(e.getActionCommand()) ) {
                
                if ( methodsTable.isEditing() ) 
                    methodsTable.getCellEditor().stopCellEditing();
                
                // User has clicked the OK button
                // check out what has changes have been made
                // and make same changes into analyzemodel.
                
                System.out.println("-------------\nMuuttuneet tiedot\n");
                // check if entity name has changed
                if (!entityName.getText().equals(selectedEntity.getName())) {
                    // name has changed
                    System.out.println("- nimi:" + selectedEntity.getName() + " --> " + entityName.getText());
                    model.getEditor().changeEntityName(selectedEntity,entityName.getText());
                }

                Set<Relation> relFrom = model.getRelationsFrom( model.getItemByName(selectedEntity.getName()) );
                
                // remove old relations(both contains and influence types) from model
                for (Relation r: relFrom) {

                    model.removeRelation(r);
                
                }
                
                for (int i = 0; i < methodsTableModel.getRowCount(); i++) {

                    // check if row has empty cells
                    if (!"".equals(methodsTableModel.getValueAt(i,0)) && !"".equals(methodsTableModel.getValueAt(i,1))) {
                    	Entity fromEntity = model.getEntityByName(selectedEntity.getName());
                    	Method method = new Method(methodsTableModel.getValueAt(i, 0).toString());
                    	Set<Relation> rel = model.getRelationsFrom(fromEntity, method);
                    	// Set<Relation> rel = model.getRelationsFrom(fromEntity, RelationType.INFLUENCE);                    	
                    	boolean methodExists = ( rel.size() > 0 );
                    	
                        // add method relation to entity
                        if ( !methodExists ) {
                        	model.addRelation(new Relation(fromEntity, RelationType.CONTAINS, method));
                        }
                        
                        // add influence type relations (entity --> method --> entity)
                        Entity toEntity = model.getEntityByName(methodsTableModel.getValueAt(i, 1).toString());
                        model.addRelation(new Relation(fromEntity, method.getName(), toEntity));
                    
                    }
                    
                }
                
                
                
                System.out.println("-------------");
                
                setVisible(false);
            }
            
            if ( "MAIN_CANCEL".equals(e.getActionCommand()) ) {
                
                // this seems to be enough to close modal dialog,
                // don't know for sure (ilari)
                setVisible(false);     
            }
            
            if ( "MAIN_DELETE_ENTITY".equals(e.getActionCommand()) ) {

                // delete the selected entity and all relations 
                // from it.. and then hide the widow
                model.removeItem(selectedEntity);
                setVisible(false);
                
            }
            
            if ( "NEW_METHOD".equals(e.getActionCommand()) ) {
                
                // add new row to method table
                Vector<String> v = new Vector<String>();
                v.add(""); 
                v.add("");
                methodsTableModel.addRow(v);
                
            }
            
            if ( "DELETE_METHOD".equals(e.getActionCommand()) ) {
                
                if (methodsTable.isEditing())
                    methodsTable.getCellEditor().cancelCellEditing();
                // delete selected row from methods table
                if (methodsTable.getSelectedRow() < 0) return;
                methodsTableModel.removeRow(methodsTable.getSelectedRow());
                
            }
            
            if ( "NEW_RELATION".equals(e.getActionCommand()) ) {
                
                // add new relation to relation table..
                Vector<String> v = new Vector<String>();
                v.add("");
                relationsTableModel.addRow(v);
                
            }

            if ( "DELETE_RELATION".equals(e.getActionCommand()) ) {
                
                relationsTable.getCellEditor().stopCellEditing();
                // delete selected row from relations table
                int row = relationsTable.getSelectedRow();
                
                if (row < 0) return; 
                relationsTableModel.removeRow(row);
                
            }
        }
        
    };
    
    public EntityPropertiesDialog(JFrame owner) throws HeadlessException {
        super(owner, "Entity Properties", true);
        
        this.setPreferredSize(new Dimension(400, 300));
        this.setLayout(new BorderLayout());
        
        // Create all required panels and set layouts for them.
        JPanel panelBottom = new JPanel();
        JPanel panelTop = new JPanel();
        panelTop.setLayout(new BorderLayout());
        JPanel panelLeft = new JPanel();
        panelLeft.setLayout(new BorderLayout());
        methodsContainerPanel = new JPanel();
        methodsContainerPanel.setLayout(new BorderLayout());
        JPanel panelLeftBottom = new JPanel();
        JPanel panelRight = new JPanel();
        panelRight.setLayout(new BorderLayout());
        relationsContainerPanel = new JPanel();
        relationsContainerPanel.setLayout(new BorderLayout());
        JPanel panelRightBottom = new JPanel();
        
        // Initialize tables and table models.
        methodsTableModel = new DefaultTableModel();
        methodsTable = new JTable(methodsTableModel);
        relationsTableModel = new DefaultTableModel();
        relationsTable = new JTable(relationsTableModel);
        methodsTable.setRowSelectionAllowed(false);
        relationsTable.setRowSelectionAllowed(false);
        
        // Initialize scroll panes for tables.
        relationsScrollPane = new JScrollPane(relationsTable);
        methodsScrollPane = new JScrollPane(methodsTable);
        
        // Divide the dialog into four sections. Top section for entity properties,
        // left (center) section for entity's methods table and bottom section for
        // general dialog buttons.
        this.getContentPane().add(panelBottom, BorderLayout.SOUTH);
        this.getContentPane().add(panelTop, BorderLayout.NORTH);
        this.getContentPane().add(panelLeft, BorderLayout.CENTER);
        // Uncomment this to invoke the other table (relations).
        // this.getContentPane().add(panelRight, BorderLayout.EAST);
        
        // Initialize relations table and its containers.
        panelRight.setPreferredSize(new Dimension(250,250));
        panelRight.add(relationsContainerPanel, BorderLayout.CENTER);
        panelRight.add(panelRightBottom, BorderLayout.SOUTH);
        panelRight.setBorder(BorderFactory.createTitledBorder("Relations"));
        relationsContainerPanel.add(relationsScrollPane, BorderLayout.NORTH);
        // Initialize methods table and its containers.
        panelLeft.add(methodsContainerPanel, BorderLayout.CENTER);
        panelLeft.add(panelLeftBottom, BorderLayout.SOUTH);
        panelLeft.setBorder(BorderFactory.createTitledBorder("Methods"));
        methodsContainerPanel.add(methodsScrollPane, BorderLayout.NORTH);
        // Initialize entity properties' container.
        panelTop.setBorder(BorderFactory.createTitledBorder("Entity Properties"));
        panelTop.add(entityName);
        
        // Create required buttons. -->
        JButton b = new JButton("New Method");
        b.setActionCommand("NEW_METHOD");
        b.addActionListener(buttonListener);
        panelLeftBottom.add(b);
        
        b = new JButton("Delete Method");
        b.setActionCommand("DELETE_METHOD");
        b.addActionListener(buttonListener);
        panelLeftBottom.add(b);
        
        b = new JButton("New Relation");
        b.setActionCommand("NEW_RELATION");
        b.addActionListener(buttonListener);
        panelRightBottom.add(b);
        
        b = new JButton("Delete Relation");
        b.setActionCommand("DELETE_RELATION");
        b.addActionListener(buttonListener);
        panelRightBottom.add(b);
                
        b = new JButton("OK");
        b.setActionCommand("MAIN_OK");
        b.addActionListener(buttonListener);
        panelBottom.add(b);

        b = new JButton("Cancel");
        b.setActionCommand("MAIN_CANCEL");
        b.addActionListener(buttonListener);
        panelBottom.add(b);

        b = new JButton("Delete Entity");
        b.setActionCommand("MAIN_DELETE_ENTITY");
        b.addActionListener(buttonListener);
        panelBottom.add(b);
        // <-- Button creation section ends here.
        
        // It is easier to size the tables if the whole dialog is of fixed size.
        this.setResizable(false);
        
        this.pack();
    }
    
    /**
     * Method to show entityPropertiesDialog 
     * @param entity entity to edit
     * @param model analyzeModel to modify
     * @return modified analyzeModel
     */
    public AnalyzeModel modifyEntityProperties(Entity entity, AnalyzeModel model) {

        this.selectedEntity = entity;
        this.model = model;
        
        initializeDialog();
        
        Output out = new Output();
        // this shows the modal dialog, and stops here until
        // the dialog is closed.        
        out.output(this.model,null);
        this.setVisible(true);
        out.output(this.model,null);
        
        // the analyzemodel is modified when OK button is clicked
        // in the buttonListener.
        
        // return the model
        return this.model;
    }
    
    private void initializeDialog() {
    	// Initialize table models by creating the required number of columns.
        if ( methodsTableModel.getColumnCount() != methodsTableColumns.length )
        	for ( int i = 0; i < methodsTableColumns.length; i++ )
        		methodsTableModel.addColumn(methodsTableColumns[i]);
        if ( relationsTableModel.getColumnCount() != relationsTableColumns.length )
        	for ( int i = 0; i < relationsTableColumns.length; i++ )
        		relationsTableModel.addColumn(relationsTableColumns[i]);
    	// Create a JTable cell editor for relations from the available entities.
        TableColumn relationsColumn = relationsTable.getColumnModel().getColumn(0);
        TableColumn methodsColumn = methodsTable.getColumnModel().getColumn(1);
        Set<Item> entities = model.getItemsType("entity");
        String[] entitiesArray = new String[entities.size()];
        int x = 0;
        // Add available entities to entitiesArray which is used for generating
        // the ComboBox editor.
        for ( Item i: entities ) entitiesArray[x++] = i.getName();
        relationsColumn.setCellEditor(createComboBoxCellEditor(entitiesArray));
        methodsColumn.setCellEditor(createComboBoxCellEditor(entitiesArray));
        // DEBUG:
        // System.out.println(Arrays.asList(entitiesArray));
        
        entityName.setText(selectedEntity.getName());
        // Erase all data from old table models.
        for ( int i = methodsTableModel.getRowCount()-1; i >= 0; i-- )
        	methodsTableModel.removeRow(i);
        for ( int i = relationsTableModel.getRowCount()-1; i >= 0; i-- )
        	relationsTableModel.removeRow(i);
        
        // Load table data from analyze model.
        //Set<Relation> targets = model.getRelationsFrom(selectedEntity, RelationType.INFLUENCE);
        Set<Relation> methods = model.getRelationsFrom(selectedEntity, RelationType.CONTAINS);
        //Iterator<Relation> j = targets.iterator();
        // Add methods to the methods table.
        for ( Relation i: methods ) {
        	Set<Relation> targets = model.getRelationsFrom(selectedEntity, (Method)i.getTo());
        	for ( Relation j: targets ) {
        		Vector<String> row = new Vector<String>();
        		row.addElement(i.getTo().getName());
        		row.addElement(j.getTo().getName());
        		methodsTableModel.addRow(row);
        	}
        }
        
        // JTables act in a quite extraordinary way what comes to sizing.
        // It's easiest to just create a fixed size dialog and statically
        // size the JTable by the dimensions of its container element.
        methodsScrollPane.setPreferredSize(methodsContainerPanel.getSize());
        methodsTable.invalidate();
        relationsScrollPane.setPreferredSize(relationsContainerPanel.getSize());
        relationsTable.invalidate();
    }
}
