package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.DefaultListModel;
import javax.swing.ListSelectionModel;

import ucot.input.UseCaseCollection;
import ucot.input.UseCase;

/**
 * Dialog for selecting use cases from given use case collection.
 * 
 * Usage:
 *   call method:
 *     selectUseCase(UseCaseCollection collection)
 *   which returns the selected use case in the given collection.
 *   
 * @author tujupien
 */
public class SelectUseCaseDialog extends JDialog {
	private DefaultListModel listModel;
	private JList list;
	private JScrollPane listScroller;
	private JPanel listContainer;
	
	/**
	 * Button action listeners.
	 */
	ActionListener buttonListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
            if ( "OK".equals(e.getActionCommand()) ) {
            	setVisible(false);
            }
            
            if ( "CANCEL".equals(e.getActionCommand()) ) {
            	list.setSelectedIndex(-1);
            	setVisible(false);
            }
		}
	};
	
	/**
	 * Constructor for SelectUseCaseDialog. The dialog is partially
	 * initialized here, but dynamic information will be initialized
	 * when the dialog is spawned with the selectUseCase method.
	 * @param owner Owner object of this dialog.
	 * @throws HeadlessException If something headless happens, an
	 *   exception is thrown. ;)
	 */
	public SelectUseCaseDialog(JFrame owner) throws HeadlessException {
        super(owner, "Select Use Case", true);
        
        // Set layout.
        this.setPreferredSize(new Dimension(250, 300));
        this.setLayout(new BorderLayout());
        
        // Create the JPanel element which contains the list.
        listContainer = new JPanel();
        listContainer.setLayout(new BorderLayout());
        this.getContentPane().add(listContainer, BorderLayout.CENTER);
        
        // Create a JPanel for buttons at the bottom of the dialog.
        JPanel panelBottom = new JPanel();
        this.getContentPane().add(panelBottom, BorderLayout.SOUTH);
        
        // Create list model and add the list in a JScrollPane to
        // the list container.
        listModel = new DefaultListModel();
		list = new JList(listModel);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        listScroller = new JScrollPane(list);
        listContainer.add(listScroller);

        // Create necessary control buttons -->
        JButton b = new JButton("OK");
        b.setActionCommand("OK");
        b.addActionListener(buttonListener);
        panelBottom.add(b);
        
        b = new JButton("Cancel");
        b.setActionCommand("CANCEL");
        b.addActionListener(buttonListener);
        panelBottom.add(b);
        // <-- Button creation ends here.
        
        this.pack();
    }
	/**
	 * This method spawns the dialog and prompts user for the
	 * desired use case. Selected use case is returned.
	 * @param collection These are the available options for user
	 *   to select his or hers desired use case.
	 * @return selected use case.
	 * @throws Exception An exception is thrown if no use case is selected.
	 */
	public UseCase selectUseCase(UseCaseCollection collection) throws Exception {
		// Empty list model from previous spawns.
		listModel.removeAllElements();
		
		// Make sure that the given collection is not empty.
		if ( collection.getUseCaseCount() < 1 )
			throw new Exception("Use case collection is empty.");
		
		// List available use cases.
		for ( int i = 0; i < collection.getUseCaseCount(); i++ ) {
			listModel.addElement((i+1) + ". " + collection.getUseCase(i).getName());
		}
		
		// Make dialog visible and halt other execution of until
		// user makes a decision or decides to not make one.
		this.setVisible(true);
		
		// Return selected use case.
		if ( list.getSelectedIndex() >= 0 && list.getSelectedIndex() < collection.getUseCaseCount() )
			return collection.getUseCase(list.getSelectedIndex());
		
		// User did not select a use case.
		throw new Exception("No use case was selected.");
	}
}
